%--------------------------------------------------------------------------
% Linear Curve Fitting
%
% Input: data - File of the measured data
%        phi -  Basic-Functions phi_i as {'phi_1','phi_2',...,'phi_n'}
%               Example: Fitting Curve is a polynom of 2nd degree
%                        => phi = {'1' 't1*t2' 't1^2' 't2^2'}
%        var - Variables as {'t1' 't2' ...} 
%        prt  - 1 Plot of the fitting curve and the data points
%               0 No plot
% Output: R2  - Rsquared
%         Coefficients x1,...,xn of the modell function
%         Plot of the fitting curve and the data points if prt = 1
%
% Example: MLA('data.m',1,{'1' 't1' 't2' 't1*t2','t1^2' 't2^2'},{'t1' 't2'}) 
%          finds the coefficients of the best fitting 2nd order polynomial
%          function of two variables regarding the measured data in the 
%          file data.m
%          
%--------------------------------------------------------------------------
function [x,bfc] = MLA(data,prt,phi,var)
disp(' ');
disp(['Multiple Curve Fitting - Start -> ',char(datestr(now))]);
disp(' ');
if ischar(data)
   in = load(data);
else
   in = data;
end
m = length(in);
q = length(phi);
lvar = length(var);
%Generating strings for the eval-command. The inline function is
%assembled depending on the number of variables of the input -> var
s1 = 'phij = inline(char(phi(j)),';
for k = 1:lvar
    if k < lvar 
       s1 = strcat(s1,'''',char(var(k)),'''',',');
    else
       s1 = strcat(s1,'''',char(var(k)),'''',');'); 
    end
end
s2 = 'A(i,j) = phij(';
for k = 1:lvar
    if k < lvar
       s2 = strcat(s2,'in(i,',num2str(k),')',',');
    else
       s2 = strcat(s2,'in(i,',num2str(k),')',');');  
    end
end
%Generate Matrix A
A = zeros(m,q);
for j = 1:q 
    eval(s1);
    for i = 1:m
        eval(s2);
    end
end
%Generate b
b = in(:,lvar+1);
%Solve the problem using Backslash-Operator
x = A\b;
%RSquared
r2 = norm(A*x - mean(b))^2/norm(b - mean(b))^2;
display(' ');
display(['RSquared: ', num2str(r2)])
display(' ');
%Formatting the output
C = '';
F = '(';
for i = 1:q
    v1 = num2str(i);
    v2 = num2str(x(i),16);
    if i == 1
       C = strcat('x',v1,' =  ',v2);
       F = strcat(F,v2,')*',char(phi(i)));
    else
       C = strcat(C,'; x',v1,' = ',v2);
       F = strcat(F,' + (',v2,')*',char(phi(i)));
     end
end
%Represent the best fitting curve as inline function
bfc = inline(F);
%Format the output
display(['Parameter: ',C]);
display(' ');
s1 = 'Best fitting curve: f(';
for k = 1:lvar
    if k < lvar
       s1 = strcat(s1,char(var(k)),',');
    else
       s1 = strcat(s1,char(var(k)),') = ',F);  
    end
end
disp(s1);
%Compute R2
s2 = 'fdat(i) = bfc(';
for k = 1:lvar
    if k < lvar
       s2 = strcat(s2,'in(i,',num2str(k),')',',');
    else
       s2 = strcat(s2,'in(i,',num2str(k),')',');');  
    end
end
for i = 1:m
    eval(s2);
end
%Plot the points of the dataset and the best fitting curve
if lvar > 2 
   prt = 0;
end
if prt == 1 
   %Generate points for plotting using the best fitting curve
   %Find the Minimum and Maximum of the t-values of input data
   if lvar == 1 
      min12 = min(in);
      max12 = max(in);
      t = (min12(1):0.1:max12(1));
      [~,l]=size(t);
      fdat = [];
      for p = l:-1:1
          fdat(p) = bfc(t(p));
      end 
      plot(in(:,1),in(:,2),'o',t,fdat,'-')
      title(['RSquared: ', v]);
   else
      min12 = min(in);
      max12 = max(in);
      hold on
      plot3(in(:,1),in(:,2),in(:,3),'.b','MarkerSize',15);
      ezmesh(bfc,[min12(1),max12(1),min12(2),max12(2)],70);
      title(['RSquared: ', v]);
      view(39,29);
      shading interp;
      colormap default;
      grid on
      hold off
   end
end