%--------------------------------------------------------------------------
% Linear Curve Fitting
%
% Input: data - File of the measured data
%           prt    - 1 Plot of the fitting curve and the data points
%                      0 No plot
%           phi   -  Basic-Functions phi_i as {'phi_1','phi_2',...,'phi_n'}
%                      Example: Fitting Curve is a polynom of 2nd degree
%                                      => phi = {'1','t','t^2'}
%
% Output: R2  - Rsquared
%             Coefficients x1,...,xn of the model function, bfc,
%             Plot of the fitting curve and the data points if prt = 1
%
% Example: LA('data_01.m',1,{'1','t','t^2'}) finds the coefficients of the
%                 best fitting 2nd order polynomial function regarding the 
%                 measured data in the file data_01.m
%--------------------------------------------------------------------------
function [x,bfc] = LA(data,prt,phi)
displ(' ');
disp(['Curve Fitting - Start -> ',char(datestr(now))]);
disp(' ');
in = load(data);
q = length(phi);
m = length(in);
%Generate Matrix A
A = zeros(m,q);
for j = 1:q
    for i = 1:m
        phij = inline(char(phi(j))); 
        A(i,j) = phij(in(i,1));
    end
end
%Generate b
b = in(:,2);
%Solve the problem using QR-Decomposition                                  
x = LAQR(A,b);
 
%Format the output
C = '';
F = '(';
for i = 1:q
    v1 = num2str(i);
    v2 = num2str(x(i));
    if i == 1
       C = strcat('x',v1,' =  ',v2);
       F = strcat(F,v2,')*',char(phi(i)));
    else
       C = strcat(C,'; x',v1,' = ',v2);
       F = strcat(F,' + (',v2,')*',char(phi(i)));
     end
end
%Represent the best fitting curve as inline function
bfc = inline(F);
%Format the output
display(['Parameter: ',C]);
display(' ');
display(['Best fitting curve: f(t) = ',F]);
%Compute R2
for i = 1:m
    fdat(i) = bfc(in(i,1));  
end 
r2 = RS(in,fdat);
v = num2str(r2);
display(' ');
display(['RSquared: ', v])
display(' ');
%Plot the points of the dataset and the best fitting curve
if prt == 1 
   %Generate points for plotting using the best fitting curve
   %Find the Minimum and Maximum of the t-values of input data
   min12 = min(in); 
   max12 = max(in);
 
   
   t = (min12(1): 0.1: max12(1));                                          
   [k,l] = size(t);                                                                             
   for p = l:-1:1
        fdat(p) = bfc(t(p));
   end 
   plot(in(:,1),in(:,2),'o',t,fdat,'-')
   grid on
end 
