%--------------------------------------------------------------------------
% Linear Curve Fitting for Basis Functions having max. 4 Var.
%
% Input: data - File of the measured data
%           prt  - 1 Plot of the fitting curve and the data points
%                    0 No plot
%           phi -  Basic-Functions phi_i as {'phi_1','phi_2',...,'phi_n'}
%                    Example: Fitting Curve is a polynom of 2nd degree
%                                    of two variables
%                                    => phi = {'1' 't1*t2' 't1^2' 't2^2'}
%           var - Variables {'t1' 't2'} 
%
% Output: R2  - Rsquared
%             Coefficients x1,...,xn of the model function
%             Plot of the fitting curve and the data points if prt = 1
%
% Example: MLA4('data.m',1,{'1' 't1' 't2' 't1*t2','t1^2' 't2^2},{'t1' 't2'}) 
%          finds the coefficients of the best fitting 2nd order polynomial
%          function of two variables regarding the measured data in the 
%          file data.m
%          
%--------------------------------------------------------------------------
function [x,bfc] = MLA4(data,prt,phi,var)
disp(' ');
disp(['Multiple Curve Fitting - Start -> ',char(datestr(now))]);
disp(' ');
in = load(data);
q = length(phi);
m = length(in);
lvar = length(var);
%Generate Matrix A
A = zeros(m,q);
for j = 1:q
    for i = 1:m
        switch lvar
            case 1
              phij = inline(char(phi(j)),char(var(1)));
              A(i,j) = phij(in(i,1));
          case 2   
              phij = inline(char(phi(j)),char(var(1)),char(var(2)));
              A(i,j) = phij(in(i,1),in(i,2));
           case 3
              phij = inline(char(phi(j)),char(var(1)),char(var(2)),...           
                            char(var(3)));
              A(i,j) = phij(in(i,1),in(i,2),in(i,3));
           case 4
              phij = inline(char(phi(j)),char(var(1)),char(var(2)),...
                            char(var(3)),char(var(4)));
              A(i,j) = phij(in(i,1),in(i,2),in(i,3),in(i,4));
        end
    end
end
%Generate b
b = in(:,lvar+1);
%Solve the problem using QR-Decomposition
x = LAQR(A,b);
%Formatting the output
C = '';
F = '(';
for i = 1:q
    v1 = num2str(i);
    v2 = num2str(x(i));
    if i == 1
       C = strcat('x',v1,' =  ',v2);
       F = strcat(F,v2,')*',char(phi(i)));
    else
       C = strcat(C,'; x',v1,' = ',v2);
       F = strcat(F,' + (',v2,')*',char(phi(i)));
     end
end
%Represent the best fitting curve as inline function
bfc = inline(F);
%Format the output
disp(['Parameter: ',C]);
disp(' ');
 
switch lvar
    case 1 
      display(['Best fitting curve: f(',char(var(1)),') = ',F]);
    case 2 
      display(['Best fitting curve: f(',char(var(1)),',',char(var(2)),') = ',F]);
    case 3 
      display(['Best fitting curve: f(',char(var(1)),',',char(var(2)),...
               ',',char(var(3)),') = ',F]);
    case 4 
      display(['Best fitting curve: f(',char(var(1)),',',char(var(2)),...
               ',',char(var(3)),',',char(var(4)),') = ',F]);
end 
%Compute R2
for i = 1:m
     switch lvar
         case 1
           fdat(i) = bfc(in(i,1)); 
         case 2
           fdat(i) = bfc(in(i,1),in(i,2));
         case 3
           fdat(i) = bfc(in(i,1),in(i,2),in(i,3));
         case 4
           fdat(i) = bfc(in(i,1),in(i,2),in(i,3),in(i,4));
     end
end
r2 = RS(in,fdat);
v = num2str(r2);
disp(' ');
disp(['RSquared: ', v])
disp(' ');
%Plot the points of the dataset and the best fitting curve
if lvar > 2 
   prt = 0
end
if prt == 1 
   %Generate points for plotting using the best fitting curve
   %Find the Minimum and Maximum of the t-values of input data
   min12 = min(in);
 
max12 = max(in);
   hold on
   plot3(in(:,1),in(:,2),in(:,3),'.','MarkerSize',15);
   ezmesh(bfc,[min12(1),max12(1),min12(2),max12(2)],70);            
   title(['RSquared: ', v]);
   view(39,29);
   shading interp;
   colormap hot;
   grid on
   hold off
end 
